/*
    Enchanced Regular User Privileges (erup)
    idea by Lukasz Wojtow <lw@wszia.edu.pl>
    code by Rafal Wijata http://www.wijata.com
*/
#ifndef __K_ERUP_H_
#define __K_ERUP_H_
/*
#if CONFIG_MODVERSIONS
#define MODVERSIONS
#include <linux/modversions.h>
#endif
*/
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/unistd.h>
#include <linux/types.h>
#include <linux/vmalloc.h>
#include <linux/smp_lock.h>
#include <linux/version.h>
#include <linux/string.h>
#include <linux/quota.h>

#define ERUP_VERSION    "0.9.8"

/* fix config deps */
#ifndef ERUP_CHROOT
#undef ERUP_CHECK_PATH
#endif
#ifndef ERUP_CHECK_PATH
#undef ERUP_CHECK_PATH_STRICT
#undef ERUP_CHECK_SYMLINK
#endif

/* assume we run >= 2.4 kernel */
#define HAVE32ID
#define HAVESNPRINTF
/* check kernel version */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
#warning compiling for kernel > 2.4, designed for 2.4
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
#warning compiling for kernel < 2.4, designed for 2.4
#undef HAVE32ID
#undef HAVESNPRINTF
#endif
/* we need /proc */
#ifndef CONFIG_PROC_FS
#error /proc fs required, but CONFIG_PROC_FS is unset
#endif

/* check for new quota code (maybe it should be in erup_core.c) */
#if defined(QFMT_VFS_V0) || defined(QFMT_VFS_OLD)
#include <linux/quotacompat.h>
#define QUOTA_READ(c) (c==Q_GETQUOTA || c==Q_XGETQUOTA\
		|| c==Q_V1_GETQUOTA || c==Q_V2_GETQUOTA)
#define QUOTA_WRITE(c) (c==Q_SETQUOTA || c==Q_XSETQLIM\
		|| c==Q_V1_SETQUOTA || c==Q_V1_SETUSE || c==Q_V1_SETQLIM\
		|| c==Q_V2_SETQUOTA || c==Q_V2_SETUSE || c==Q_V2_SETQLIM)
#else
#define QUOTA_READ(c) (c==Q_GETQUOTA)
#define QUOTA_WRITE(c) (c==Q_SETQUOTA || c==Q_SETUSE || c==Q_SETQLIM)
#endif

#define DEVICENAME	"erup"

#define NOERR		0

/* assume gid_t == uid_t */
typedef uid_t		id_t;
#ifdef HAVE32ID
typedef old_uid_t	old_id_t;
#endif

/* id may switch to min <= ?ID <= max
   used for setuid, setgid, quotactl, setgroups */
struct ug_entry {
    id_t		min;
    id_t		max;
    struct ug_entry	*next;
};
struct ug_list {
    id_t		id;
    struct ug_entry	*entry;
    struct ug_list	*next;
};

/* id may chroot within path
   used for chroot() */
struct ch_entry {
    int			pathlen;	/* for speed only */
    char		*path;
    struct ch_entry	*next;
};
struct ch_list {
    id_t		id;
    struct ch_entry	*entry;
    struct ch_list	*next;
};

int is_allowed_chr(id_t id, const char *path, struct dentry *dp);
int is_allowed_uid(id_t id, id_t to);
int is_allowed_gid(id_t id, id_t to);
int is_allowed_quota(id_t id, int cmd, id_t to_id);
int is_allowed_groups(id_t id, int size);
int erup_init(void);
int erup_finish(void);

#endif
